<?php
//
// LMS-API version 0,1,0
//
// Copyright 2002 DuckCorp and RtpNet
//
// This program is free software ; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation (http://www.gnu.org/).
//



/**
* Number of Attributs types
*
* @const LMS_MAX_ATYPE
*/
define ('LMS_MAX_ATYPE', 6);

/**
* Attribut type : unknown
*
* @const LMS_ATYPE_UNKNOWN
*/
define ('LMS_ATYPE_UNKNOWN', 0);
/**
* Attribut type : integer
*
* @const LMS_ATYPE_INT
*/
define ('LMS_ATYPE_INT', 1);
/**
* Attribut type : string
*
* @const LMS_ATYPE_STR
*/
define ('LMS_ATYPE_STR', 2);
/**
* Attribut type : boolean
*
* @const LMS_ATYPE_BOOL
*/
define ('LMS_ATYPE_BOOL', 3);
/**
* Attribut type : telephone number
*
* @const LMS_ATYPE_TELNO
*/
define ('LMS_ATYPE_TELNO', 4);
/**
* Attribut type : mac address
*
* @const LMS_ATYPE_MAC
*/
define ('LMS_ATYPE_MAC', 5);
/**
* Attribut type : ip address
*
* @const LMS_ATYPE_IP
*/
define ('LMS_ATYPE_IP', 6);

/**
* LMS Object Base Class
*
* @access	private
*/
class LmsObjectManager extends LmsRoot
{
	/**
	* List of all Object Attributes (with attributes parameters)
	*
	* @access	private
	* @var		array
	*/
	var	$_attrs;
	/**
	* LDAP Connection
	*
	* @access	private
	* @var		object LmsLdapConnection
	*/
	var	$_lc;
	/**
	* LDAP Entries
	*
	* @access	private
	* @var		object LmsLdapEntries
	*/
	var	$_ent;
	/**
	* DN of ths selected result or "" if a new object
	*
	* @access	private
	* @var		string
	*/
	var	$_dn;
	/**
	* Number of the Result currently used
	*
	* @access	private
	* @var		integer
	*/
	var	$_pos;
	/**
	* Attribute used to create the DN
	*
	* @access	private
	* @var		string
	*/
	var	$_dn_attr;
	/**
	* Root of this sort of Objects
	*
	* @access	private
	* @var		string
	*/
	var	$_root_dn;
	/**
	* List of keys in which to propagate Object renaming
	*
	* @access	private
	* @var		array
	*/
	var	$_propagation_key_list;
	/**
	* List of keys needed for check
	*
	* @access	private
	* @var		array
	*/
	var	$_check_key_list;

	/**
	* Constructor
	*
	* @access	public
	* @param	object LmsLdapConnection	&$ldap_connection
	*/
	function LmsObjectManager(&$ldap_connection)
	{
		LmsRoot::LmsRoot();

		$this->_attrs = array();
		$this->_lc = $ldap_connection;
		$this->_ent =& new LmsLdapEntries();
		$this->_dn = "";
		$this->_pos = 0;
		$this->_propagation_key_list = array();
		$this->_check_key_list = array();

		$this->_register_attr("Object Class", "objectclass", LMS_ATYPE_STR, 30, 15, true, false);

		register_shutdown_function(array(&$this, "_LmsObjectManager"));
	}

	/**
	* Destructor
	*
	* @access	private
	*/
	function _LmsObjectManager()
	{
		unset($this->_ent);
	}

	/**
	* Give DN of the current result
	*
	* @access	public
	* @return	string
	*/
	function get_dn()
	{
		return $this->_dn;
	}

	/**
	* Set Name change of the current result (for rename remembering purpose)
	*
	* @access	private
	* @param	string		$new_dn		New DN
	*/
	function _set_name_change($new_dn)
	{
		$this->_dn = $new_dn;
		$this->_ent->set_dn($this->_pos, $new_dn);
	}

	/**
	* Register a new Attribut for this object
	*
	* @access	private
	* @param	string		$name		Human name of this Attribut
	* @param	string		$key		Key to access this Attribut
	* @param	string		$type		Value-type of this Attribut
	* @param	integer		$maxlength	Max length of this Attribut (for string values for eg)
	* @param	integer		$maxvalues	Maximum number of values for this Attribut
	* @param	boolean		$required	Is this Attribut required ?
	* @param	boolean		$changeable	Is this Attribut changeable ?
	* @return	boolean		$ok		True if successful
	*/
	function _register_attr($name, $key, $type, $maxlength, $maxvalues, $required = false, $changeable = true)
	{
		if ($type > LMS_MAX_ATYPE)
			return false;
		if (!is_bool($required))
			return false;
		if (!is_bool($changeable))
			return false;

		$this->_attrs[$key]['value'] = "";
		$this->_attrs[$key]['oldvalue'] = "";
		$this->_attrs[$key]['name'] = $name;
		$this->_attrs[$key]['type'] = $type;
		$this->_attrs[$key]['maxlenghth'] = $maxlength;
		$this->_attrs[$key]['maxvalues'] = $maxvalues;
		$this->_attrs[$key]['required'] = $required;
		$this->_attrs[$key]['changeable'] = $changeable;
		$this->_attrs[$key]['changed'] = false;

		return true;
	}

	/**
	* Give the value of an Attribut
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	mixed
	*/
	function get_attr_value($key)
	{
		return $this->_attrs[$key]['value'];
	}

	/**
	* Give the old value of an Attribut
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	mixed
	*/
	function get_attr_oldvalue($key)
	{
		return $this->_attrs[$key]['oldvalue'];
	}

	/**
	* Give the name of an Attribut
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	string
	*/
	function get_attr_name($key)
	{
		return $this->_attrs[$key]['name'];
	}

	/**
	* Give the type of an Attribut
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	integer
	*/
	function get_attr_type($key)
	{
		return $this->_attrs[$key]['type'];
	}

	/**
	* Give maximum of values for an Attribut
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	integer
	*/
	function get_attr_maxvalues($key)
	{
		return $this->_attrs[$key]['maxvalues'];
	}

	/**
	* Give the maximum length of the value of an Attribut
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	integer
	*/
	function get_attr_maxlength($key)
	{
		return $this->_attrs[$key]['maxlength'];
	}

	/**
	* Tell is the Attribut is required
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	boolean
	*/
	function is_attr_required($key)
	{
		return $this->_attrs[$key]['required'];
	}

	/**
	* Tell if the Attribut is changeable
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	boolean
	*/
	function is_attr_changeable($key)
	{
		return $this->_attrs[$key]['changeable'];
	}

	/**
	* Tell is the Attribut has changed
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @return	boolean
	*/
	function has_attr_changed($key)
	{
		return $this->_attrs[$key]['changed'];
	}

	/**
	* Set value of an Attribut
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @param	mixed		$value		Value of the Attribut
	* @return	boolean		$ok		True if successful
	*/
	function set_attr_value($key, $value)
	{
		if (!$this->_attrs[$key]['changeable'])
			return false;

		return $this->_set_attr_value($key, $value);
	}

	/**
	* Set value of an Attribut (Internal)
	*
	* @access	private
	* @param	string		$key		Key of the Attribut
	* @param	mixed		$value		Value of the Attribut
	* @return	boolean		$ok		True if successful
	*/
	function _set_attr_value($key, $value)
	{
		if ($this->_check_attr_value($key, $value))
		{
			if (($this->_attrs[$key]['maxvalues'] == 1) && (is_array($value)))
				return false;
			if (($this->_attrs[$key]['maxvalues'] > 1) && (!is_array($value)))
				return false;
			if ($this->_attrs[$key]['value'] != $value)
				$this->_attrs[$key]['changed'] = true;
			if (!$this->_attrs[$key]['changed'])
				$this->_attrs[$key]['oldvalue'] = $this->_attrs[$key]['value'];
			$this->_attrs[$key]['value'] = $value;
			return true;
		}

		return false;
	}

	/**
	* Check value before setting an Attribut
	*
	* @access	private
	* @param	string		$key		Key of the Attribut
	* @param	mixed		$value		Value of the Attribut
	* @return	boolean		$ok		True if successful
	*/
	function _check_attr_value($key, $value)
	{
		$values = $this->_attrs[$key]['value'];

		if (!is_array($values))
			$values = array($values);

		for ($i=0; $i<count($values); $i++)
		{
			$val = $values[$i];

			switch ($this->_attrs[$key]['type'])
			{
			case LMS_ATYPE_INT:
				if (!is_numeric($val))
					return false;
				break;
			case LMS_ATYPE_STR:
				if (!is_string($val))
					return false;
				break;
			case LMS_ATYPE_BOOL:
				if (!is_bool($val))
					return false;
				break;
			case LMS_ATYPE_TELNO:
				// International Telephone Number only
				if (preg_match("/^\+\d{8,15}$/", $val)<=0)
					return false;
				break;
			case LMS_ATYPE_MAC:
				if (preg_match("/^[0-9a-z]{2}:[0-9a-z]{2}:[0-9a-z]{2}:[0-9a-z]{2}:[0-9a-z]{2}:[0-9a-z]{2}$/i", $val)<=0)
					return false;
				break;
			case LMS_ATYPE_IP:
				if (preg_match("/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/i", $val)<=0)
					return false;
				break;
			default:
				return false;
			}
		}

		return true;
	}

	/**
	* Convert an LDAP Entry to Attributs
	*
	* @access	private
	* @param	integer		$i		Number of the Entry to convert
	*/
	function _entry_to_attrs($i = 0)
	{
		if (($i<0) || ($i >= $this->_ent->get_count()))
			$i = 0;
		$tab = $this->_ent->get_entry($i);

		foreach ($this->_attrs as $key => $val)
		{
			if ($this->_attrs[$key]['maxvalues'] == 1)
				$this->_attrs[$key]['value'] = $tab[$key][0];
			else
				$this->_attrs[$key]['value'] = $tab[$key];
			$this->_attrs[$key]['oldvalue'] = $this->_attrs[$key]['value'];
		}

		$this->_dn = $this->_ent->get_dn($i);
		$this->_pos = $i;
	}

	/**
	* Activete Attributes (used for Dynamic Objects)
	*
	* @access	private
	* @param	array		$tab		List of Attributs keys
	*/
	function _activate_attrs($tab, $do_clean = false)
	{
		foreach ($tab as $key)
		{
			$this->_attrs[$key]['changeable'] = true;
			if ($do_clean)
				$this->_attrs[$key]['value'] = "";
		}
	}

	/**
	* Desactivete Attributes (used for Dynamic Objects)
	*
	* @access	private
	* @param	array		$tab		List of Attributs keys
	*/
	function _desactivate_attrs($tab)
	{
		foreach ($tab as $key)
		{
			$this->_attrs[$key]['changeable'] = false;
			$this->_attrs[$key]['value'] = null;
		}
	}

	/**
	* Propagate Attributs changes into an LDAP Entry
	*
	* @access	private
	* @return	array		$mod		List of changed Attributs
	*/
	function _attrs_changes_to_current_entry()
	{
		$mod = array();
		$tab =& $this->_ent->get_entry($this->_pos);
		foreach ($this->_attrs as $key => $val)
		{
			if ($this->_attrs[$key]['value']==null)
				unset($tab[$key][0]);
			else if ($this->_attrs[$key]['changed'])
			{
				$mod[] = $key;
				if ($this->_attrs[$key]['maxvalues']>1)
					$tab[$key] = $this->_attrs[$key]['value'];
				else
					$tab[$key][0] = $this->_attrs[$key]['value'];
			}
		}
		return $mod;
	}

	/**
	* Has Attributs changed
	*
	* @access	private
	* @return	array		$mod		List of changed Attributs
	*/
	function _attrs_changed()
	{
		$mod = array();
		foreach ($this->_attrs as $key => $val)
		{
			if ($this->_attrs[$key]['changed'])
				$mod[] = $key;
		}
		return $mod;
	}

	/**
	* Clear all the Attributs
	*
	* @access	public
	*/
	function clear()
	{
		foreach ($this->_attrs as $key => $val)
		{
			$this->_attrs[$key]['oldvalue'] = null;
			$this->_attrs[$key]['value'] = null;
			$this->_attrs[$key]['changed'] = false;
		}
		$this->_dn = "";
		$this->_pos = 0;
		$this->_ent->clear();
	}

  	/**
	* Display an Object content, for debugging purpose
	*
	* @access	public
	*/
	function display_object()
	{
		$this->display_array($this->_attrs);
	}

	/**
	* Select a result
	*
	* @access	public
	* @param	integer		$i		Number of the result to select
	*/
	function select_result($i)
	{
		$this->_entry_to_attrs($i);
	}

	/**
	* Give the number of results
	*
	* @access	public
	* @return	integer
	*/
	function get_result_number()
	{
		return $this->_ent->get_count();
	}

	/**
	* Give the number of ths selected result
	*
	* @access	public
	* @return	integer
	*/
	function get_current_pos()
	{
		return $this->_pos;
	}

	/**
	* Select next result
	*
	* @access	public
	*/
	function next_result()
	{
		$this->select_result($this->_pos+1);
	}

	/**
	* Select previous result
	*
	* @access	public
	*/
	function previous_result()
	{
		$this->select_result($this->_pos-1);
	}

	/**
	* Sort results
	*
	* @access	public
	* @param	string		$attr1		First key for sorting
	* @param	string		$attr2		Second key for sorting
	*/
	function sort_result_by_attr($attr1, $attr2 = null)
	{
		$this->_ent->sort_entries_by_attr($attr1, $attr2);
	}

	/**
	* Delete an Object
	*
	* @access	public
	* @param	string		$name		Name of Oject to delete
	* @return	boolean		$ok		True if successful
	*/
	function delete($name = "")
	{
		if ($name)
		{
			if (!$this->_lc->search($this->_root_dn, $this->_dn_attr."=".$name, $this->_ent, array($this->_dn_attr)))
				return false;
			if ($this->_ent->get_count()<=0)
				return false;
			$this->_entry_to_attrs(0);
		}
		$object = $this->get_dn();
		if (!$object)
			return false;
		
		return $this->_lc->delete($object);
	}

	/**
	* Rename an Object (!!!DISACTIVATED!!!)
	*
	* @access	public
	* @param	string		$new_name	New name of Oject to delete
	* @param	string		$name		Name of Oject to delete
	* @return	boolean		$ok		True if successful
	*/
	function rename($name, $new_name)
	{
		if (!$name || !$new_name)
			return false;
		if (!$this->_lc->search($this->_root_dn, $this->_dn_attr."=".$name, $this->_ent, $this->_check_key_list))
			return false;
		if ($this->_ent->get_count()==0)
			return false;

		$this->_entry_to_attrs(0);

		$this->set_attr_value($this->_dn_attr, $new_name);

		return $this->commit_changes();
	}

	/**
	* Save the changes in LDAP Database
	*
	* @access	public
	* @return	boolean		$ok		True if successful
	*/
	function commit_changes()
	{
		if ($this->_ent->get_count() == 0)
			return false;

		$mod = $this->_attrs_changed();
		if (count($mod)==0)
			return false;

		$this->_pre_commit_changes();
		
		if ($this->_attrs['objectclass']['changed'])
			if (!$this->_fetch_all_attrs())
				return false;

		$mod = $this->_attrs_changes_to_current_entry();
		if (count($mod) == 0)
			return false;

		if (!$this->_check_changes($mod))
			return false;
		
		if ($this->_attrs['objectclass']['changed'])
			$ret = $this->_lc->modify($this->get_dn() ,$this->_ent, $this->_pos);
		else
			$ret = $this->_lc->modify($this->get_dn() ,$this->_ent, $this->_pos, $mod);

		if ($ret && ($this->_attrs[$this->_dn_attr]['changed'] || $this->_need_renaming()))
		{
			$object = $this->get_dn();
			$sub = $this->_get_sub_dn();
			if ($sub)
				$sub.=",";
			$new_name = $this->_attrs[$this->_dn_attr]['value'];
			$new_object = $this->_dn_attr."=".strtolower($new_name).",".$sub.$this->_root_dn;
			if ($new_object == $object)
				return true;

			$ret = $this->_lc->rename($object, $new_object);
			if ($ret)
			{
				$this->_set_name_change($new_object);
				$name = $this->_attrs[$this->_dn_attr]['oldvalue'];
				$this->_propagate_object_renaming($name, $new_name);
			}
		}

		return $ret;
	}

	/**
	* Propagate Key change
	*
	* @access	public
	* @param	string		$key		Key of the Attribut
	* @param	string		$old_value	Old value of Attribut
	* @param	string		$new_value	New value of Attribut
	* @return	boolean		$ok		True if successful
	*/
	function propagate_key_change($key, $old_value, $new_value)
	{
		$ent =& new LmsLdapEntries();

		$ret = $this->_lc->search($this->get_dn(), $key."=".$old_value, $ent);
		$nb = $ent->get_count();
		if (!$ret || ($nb==0))
			return false;

		for ($i=0; $i<$nb; $i++)
		{
			$tab =& $ent->get_entry($i);
			if (count($tab[$key])>1)
				$tab[key][array_search($old_value, $tab[$key])]=$new_value;
			else
				$tab[key][0]=$new_value;
		}

		return $nb;
	}

	/**
	* Propagate Object_renaming
	*
	* @access	private
	* @param	string		$key		Key of the Attribut
	* @param	string		$old_value	Old value of Attribut
	* @param	string		$new_value	New value of Attribut
	* @return	boolean		$ok		True if successful
	*/
	function _propagate_object_renaming($old_name, $new_name)
	{
		foreach ($this->_propagation_key_list as $key)
			$this->propagate_key_change($key, $old_name, $new_name);
	}

	/**
	* Add a new Userin LDAP Database
	*
	* @access	public
	* @return	boolean		$ok		True if successful
	*/
	function add()
	{
		$this->_pre_commit_changes();
		
		$mod = $this->_attrs_changes_to_current_entry();
		if (count($mod) == 0)
			return false;

		if (($this->_ent->get_count()==0) && (!$this->_attrs[objectclass][value]))
			return false;

		if (!$this->_check_changes($mod))
			return false;
	
		$sub = $this->_get_sub_dn();
		if ($sub)
			$sub.=",";
		$new_name = $this->_attrs[$this->_dn_attr]['value'];
		$new_object = $this->_dn_attr."=".strtolower($new_name).",".$sub.$this->_root_dn;
		$ret = $this->_lc->add($new_object, $this->_ent, $this->_pos);

		return $ret;
	}

	/**
	* Check if required Attributs are set
	*
	* @access	private
	* @return	boolean		$ok		True if successful
	*/
	function _check_changes($mod = array())
	{
		foreach ($this->_attrs as $key => $val)
		{
			if ((count($mod)==0) || in_array($key, $mod))
			{
				if ($this->_attrs[$key]['needed'] && !$this->_attrs[$key]['value'])
					return false;
			}
		}

		return true;
	}

	/**
	* Fetch every Attributs of the Object for the user
	*
	* @access	public
	* @return	boolean		$ok		True if successful
	*/
	function fetch_all_attrs()
	{
		$ret = $this->_fetch_all_attrs();
		if ($ret)
			$this->_entry_to_attrs($this->_pos);

		return $ret;
	}

	/**
	* Fetch every Attributs of the Object (internaly only)
	*
	* @access	private
	* @return	boolean		$ok		True if successful
	*/
	function _fetch_all_attrs()
	{
		if ($this->_ent->get_count() == 0)
			return false;

		$ent =& new LmsLdapEntries();

		$ret = $this->_lc->search($this->get_dn(), "objectClass=*", $ent);
		if (!$ret || ($ent->get_count()==0))
			return false;

		$tab =& $ent->get_entry(0);
		$tab2 =& $this->_ent->get_entry($this->_pos);
		$tab2 = $tab;

		return true;
	}

	/**
	* Do what need to be done before applying changes (TO BE OVERLOADED)
	*
	* @access	private
	*/
	function _pre_commit_changes()
	{
		;
	}

	/**
	* Additionnal Check (TO BE OVERLOADED)

	*
	* @access	private
	* @return	boolean		$ok		True if successful
	*/
	function _check_changes($mod = array())
	{
		return true;
	}

	/**
	* Return a sub DN if needed (TO BE OVERLOADED)

	*
	* @access	public
	* @return	string
	*/
	function _get_sub_dn()
	{
		return "";
	}

	/**
	* Do Object need a renaming before saving changes ? (TO BE OVERLOADED)

	*
	* @access	private
	* @return	boolean
	*/
	function _need_renaming()
	{
		return false;
	}

}

?>
