<?php
//
// LMS-API version 0,1,0
//
// Copyright 2002 DuckCorp and RtpNet
//
// This program is free software ; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation (http://www.gnu.org/).
//



/**
* LMS Root Class
*
* @access	private
* @package	LMS-API
* @author	Duck <duck@DuckCorp.org>, Rtp <rtp@rtp.dyndns.org>
* @version	0.1.0
*/
class LmsRoot
{
	/**
	* Last error message
	*
	* @access	private
	* @var		string
	*/
	var	$_last_error;

	/**
	* Should error messages be printed out ?
	*
	* @access	private
	* @var		boolean
	*/
	var	$_display_errors;

	/**
	* Constructor
	*
	* @access	public
	*/
	function LmsRoot()
	{
		$this->_last_error = "";
		$this->_display_errors = false;
		
		register_shutdown_function(array(&$this, "_LmsRoot"));
	}

	/**
	* Destructor
	*
	* @access	private
	*/
	function _LmsRoot()
	{
	}

	/**
	* Give last error message
	*
	* @access	public
	* @return	string
	*/
	function last_error()
	{
		return $this->_last_error;
	}

	/**
	* Define if error messages should be printed or not
	*
	* @access	public
	* @param	boolean		$display_errors
	*/
	function set_display_errors($display_errors)
	{
		$this->_display_errors = $display_errors;
	}

  	/**
	* Set error message, and if allowed display it
	*
	* @access	private
	* @param	string		$err
	*/
	function _set_error($err = "")
	{
		$this->_last_error = $err;

		if ($this->_display_errors)
			if ($this->_last_error)
			print("### ".$this->_last_error." ###<br>");
	}

  	/**
	* Display an array, for debugging purpose
	*
	* @access	public
	* @param	array		&$arr
	*/
	function display_array(&$arr)
	{
		if (!is_array($arr))
			return;

		echo '<table cellpadding="0" cellspacing="0" border="1">';
		foreach ($arr as $key1 => $elem1)
		{
			echo '<tr>';
			echo '<td>'.$key1.'&nbsp;</td>';
			if (is_array($elem1))
				$this->_display_array2($elem1);
			else
				echo '<td>'.$elem1.'&nbsp;</td>';
			echo '</tr>';
		}
		echo '</table>';
	}

  	/**
	* Display a part of an array
	*
	* @access	private
	* @param	array		$arr2
	* @see		display_array()
	*/
	function _display_array2($arr2)
	{
		echo '<td>';
		echo '<table cellpadding="0" cellspacing="0" border="1">';
		foreach ($arr2 as $key => $elem)
		{
			echo '<tr>';
			echo '<td>'.$key.'&nbsp;</td>';
			if (is_array($elem))
				$this->_display_array2($elem);
			else
			{
				if (is_bool($elem))
				{
					if ($elem)
						echo '<td>#true#</td>';
					else
						echo '<td>#false#</td>';
				}
				else
					echo '<td>'.htmlspecialchars($elem).'&nbsp;</td>';
			}
			echo '</tr>';
		}
		echo '</table>';
		echo '</td>';
	}

  	/**
	* Crypt for unix usage
	*
	* @access	private
	* @param	string		$passwd		Clear-text Password
	* @return	string
	*/
	function _unix_crypt($passwd)
	{
		$salt=substr(ereg_replace("[^a-zA-Z0-9./]", "", crypt(rand(10000000,99999999), rand(10,99))), 2, 2);
		return crypt($passwd, "$1$".$salt);
	}

  	/**
	* Test if is an hexadecimal number
	*
	* @access	private
	* @param	string		$val
	* @return	boolean
	*/
	function _is_hexa($val)
	{
		if (preg_match("/^[0-9a-f]*$/i", $val)>0)
			return true;
		else
			return false;
	}
}

?>
