<?php
/*

Copyright (c)2002-2003 DuckCorp(tm) and RtpNet(tm)



This file is part of LMS.

LMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

LMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LMS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/**
* Host Type : unknown
*
* @const LMS_HTYPE_UNKNOWN
*/
define('LMS_HTYPE_UNKNOWN', 0);
/**
* Host Type : common
*
* @const LMS_HTYPE_COMMON
*/
define('LMS_HTYPE_COMMON', 1);
/**
* Host Type : server
*
* @const LMS_HTYPE_SERVER
*/
define('LMS_HTYPE_SERVER', 2);

/**
* LMS Host Manager CLass
*
* @access	public
*/
class LmsHostManager extends LmsObjectManager
{

	/**
	* Constructor
	*
	* @access	public
	* @param	object LmsLdapConnection	&$ldap_connection
	*/
	function LmsHostManager(&$ldap_connection)
	{
		LmsObjectManager::LmsObjectManager(&$ldap_connection);

		$this->_dn_attr = "cn";
		$this->_root_dn = "ou=Hosts,ou=Groups,dc=coinnetwork,dc=dl";
		$this->_propagation_key_list = array('hostname', 'servername', 'primaryServername');
		$this->_check_key_list = array();

		$this->_register_attr("Host Name", "cn", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("MAC Adresses", "macaddress", LMS_ATYPE_MAC, null, 10, true);
		$this->_register_attr("IP Adresses", "iphostnumber", LMS_ATYPE_IP, null, 10, true);
		$this->_register_attr("DNS Aliases", "dnsalias", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Is a Router", "isrouter", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Is a mobile computer", "ismobilecomputer", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Owner", "owneruid", LMS_ATYPE_STR, 15, 1, true);
		$this->_register_attr("Administrator", "adminuid", LMS_ATYPE_STR, 15, 1, false);
		$this->_register_attr("Description", "description", LMS_ATYPE_STR, 300, 1, false);
		
		register_shutdown_function(array(&$this, "_LmsHostManager"));
	}

	/**
	* Destructor
	*
	* @access	public
	*/
	function _LmsHostManager()
	{
	}

	/**
	* Give Host Type
	*
	* @access	public
	* @return	integer
	*/
	function get_host_type()
	{
		if (in_array('coinnetworkCommonHost', $this->_attrs['objectclass']))
			return LMS_HTYPE_COMMON;
		if (in_array('coinnetworkServerHost', $this->_attrs['objectclass']))
			return LMS_HTYPE_SERVER;

		return LMS_HTYPE_UNKNOWN;
	}

	/**
	* Find Host
	*
	* @access	public
	* @param	string		$cn		CN of the Host
	* @return	boolean		$ok		True if successful
	*/
	function find($cn)
	{
		if ($this->_lc->search("ou=Hosts,dc=coinnetwork,dc=dl", "cn=".$cn, $this->_ent))
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Get a Host list
	*
	* Always fetched attributs : objectclass, cn
	*
	* @access	public
	* @param	integer		$c		Host Type to list (0 => every hosts)
	* @param	array		$attr		Complementary attributs to fetch
	* @param	string		$crit		Alternative criterias
	* @return	interger
	*/
	function get_list($c, $attr = array(), $crit = "")
	{
		if (!is_array($attr))
			$attr = array();
		$attr = array_merge(array('objectclass', 'cn'), $attr);
		if (!$crit)
			$crit = "cn=*";
		switch ($c)
		{
		case 0:
			$ret = $this->_lc->search("ou=Hosts,dc=coinnetwork,dc=dl", $crit, $this->_ent, $attr);
			break;
		case 1:
			$ret = $this->_lc->search("ou=CommonHosts,ou=Hosts,dc=coinnetwork,dc=dl", $crit, $this->_ent, $attr, false);
			break;
		case 2:
			$ret = $this->_lc->search("ou=ServerHosts,ou=Hosts,dc=coinnetwork,dc=dl", $crit, $this->_ent, $attr, false);
			break;
		}

		if ($ret)
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Additionnal Check (OVERLOAD)
	*
	* @access	private
	* @return	boolean		$ok		True if successful
	*/
	function _check_changes($mod = array())
	{
		$host_type = $this->get_host_type();
		if ($host_type==LMS_HTYPE_SERVER && !$this->get_attr_value('adminuid'))
			return false;

		return LmsObjectManager::_check_changes($mod);
	}

	/**
	* Return a sub DN if needed (OVERLOAD)
	*
	* @access	public
	* @return	string
	*/
	function _get_sub_dn()
	{
		$host_type = $this->get_host_type();

		switch($host_type)
		{
		case LMS_HTYPE_SERVER:
			return "ou=ServerHosts";
		default:
			return "ou=CommonHosts";
		}
	}

	/**
	* Change Host Type
	*
	* @access	public
	* @param	integer		$new_host_type		Type of Host
	* @return	boolean		$ok		True if successful
	*/
	function set_host_type($new_host_type)
	{
		if ($this->_ent->get_count() != 0)
		{
			$host_type = $this->get_host_type();

			if ($new_host_type==$host_type)
				return false;
		}

		return $this->_set_host_type($new_host_type);
	}

	/**
	* Change Host Type (internal use)
	*
	* @access	private
	* @param	integer		$new_host_type		Type of Host
	* @return	boolean		$ok		True if successful
	*/
	function _set_host_type($new_host_type = null, $set_changed = true)
	{
		if (!$new_host_type)
			$new_host_type = $this->get_host_type();

		switch($new_host_type)
		{
		case LMS_HTYPE_SERVER:
			$this->_attrs['objectclass']['value'] = array('top', 'coinnetworkServerHost');
			$this->_activate_attrs(array('adminuid'));
			break;
		case LMS_HTYPE_COMMON:
			$this->_attrs['objectclass']['value'] = array('top', 'coinnetworkCommonHost');
			$this->_desactivate_attrs(array('adminuid'));
			break;
		default:
			return false;
		}

		if ($set_changed)
			$this->_attrs['objectclass']['changed'] = true;

		return true;
	}

	/**
	* Do Object need a renaming before saving changes ? (OVERLOAD)
	*
	* @access	private
	* @return	boolean
	*/
	function _need_renaming()
	{
		return $this->_attrs['objectclass']['changed'];
	}

	/**
	* Convert an LDAP Entry to Attributs (OVERLOAD)
	*
	* @access	private
	* @param	integer		$i		Number of the Entry to convert
	*/
	function _entry_to_attrs($i)
	{
		LmsObjectManager::_entry_to_attrs($i);

		$this->_set_host_type(null, false);
	}

}

?>
