<?php
/*

Copyright (c)2002-2003 DuckCorp(tm) and RtpNet(tm)



This file is part of LMS.

LMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

LMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LMS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/**
* LMS Service Manager CLass
*
* @access	public
*/
class LmsServiceManager extends LmsObjectManager
{

	/**
	* Constructor
	*
	* @access	public
	* @param	object LmsLdapConnection	&$ldap_connection
	*/
	function LmsServiceManager(&$ldap_connection)
	{
		LmsObjectManager::LmsObjectManager(&$ldap_connection);

		$this->_dn_attr = "cn";
		$this->_root_dn = "ou=Services,dc=coinnetwork,dc=dl";
		$this->_propagation_key_list = array('suscribedService');
		$this->_check_key_list = array();

		$this->_register_attr("Common Name", "cn", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("Primary Server", "primaryservername", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("Secondary Server(s)", "servername", LMS_ATYPE_STR, 30, 15, false);
		$this->_register_attr("Ports", "ipserviceport", LMS_ATYPE_INT, null, 15, false);
		$this->_register_attr("Protocol", "ipserviceprotocol", LMS_ATYPE_PROTO, null, 1, true);
		$this->_register_attr("Service User", "serviceuid", LMS_ATYPE_STR, null, 1, false);
		$this->_register_attr("DNS Aliases", "dnsalias", LMS_ATYPE_STR, null, 1, false);
		$this->_register_attr("Description", "description", LMS_ATYPE_STR, 300, 1, false);
		
		register_shutdown_function(array(&$this, "_LmsServiceManager"));
	}

	/**
	* Destructor
	*
	* @access	public
	*/
	function _LmsServiceManager()
	{
	}

	/**
	* Find Service
	*
	* @access	public
	* @param	string		$cn		CN of the Service
	* @return	boolean		$ok		True if successful
	*/
	function find($cn)
	{
		if ($this->_lc->search("ou=Services,dc=coinnetwork,dc=dl", "cn=".$cn, $this->_ent))
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Get a Service list
	*
	* Always fetched attributs : objectclass, cn
	*
	* @access	public
	* @param	array		$attr		Complementary attributs to fetch
	* @param	string		$crit		Alternative criterias
	* @return	interger
	*/
	function get_list($attr = array(), $crit = "")
	{
		if (!is_array($attr))
			$attr = array();
		$attr = array_merge(array('objectclass', 'cn'), $attr);
		if (!$crit)
			$crit = "cn=*";

		$ret = $this->_lc->search("ou=Services,dc=coinnetwork,dc=dl", $crit, $this->_ent, $attr, false);

		if ($ret)
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

}

?>
