<?php
/*

Copyright (c)2002-2003 DuckCorp(tm) and RtpNet(tm)



This file is part of LMS.

LMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

LMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LMS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/**
* LMS Unix Group Manager CLass
*
* @access	public
*/
class LmsUnixGroupManager extends LmsObjectManager
{

	/**
	* Constructor
	*
	* @access	public
	* @param	object LmsLdapConnection	&$ldap_connection
	*/
	function LmsUnixGroupManager(&$ldap_connection)
	{
		LmsObjectManager::LmsObjectManager(&$ldap_connection);

		$this->_dn_attr = "cn";
		$this->_root_dn = "ou=UnixGroups,ou=Groups,dc=coinnetwork,dc=dl";
		$this->_propagation_key_list = array('gidNumber');
		$this->_check_key_list = array();

		$this->_register_attr("Host Name", "cn", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("GID Number", "gidnumber", LMS_ATYPE_INT, null, 1, true);
		$this->_register_attr("Password", "userpassword", LMS_ATYPE_STR, 100, 1, false);
		$this->_register_attr("Secondary Group Members", "memberuid", LMS_ATYPE_STR, 15, 100, false);
		$this->_register_attr("Description", "description", LMS_ATYPE_STR, 300, 1, false);
		
		register_shutdown_function(array(&$this, "_LmsUnixGroupManager"));
	}

	/**
	* Destructor
	*
	* @access	public
	*/
	function _LmsUnixGroupManager()
	{
	}

	/**
	* Find Group by GID
	*
	* @access	public
	* @param	string		$gid		GID of the Group
	* @return	boolean		$ok		True if successful
	*/
	function find_by_gid($gid)
	{
		if ($this->_lc->search("ou=UnixGroups,ou=Groups,dc=coinnetwork,dc=dl", "cn=".$gid, $this->_ent))
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Find Group by GID Number
	*
	* @access	public
	* @param	integer		$gidnumber	GID Number of the Group
	* @return	boolean		$ok		True if successful
	*/
	function find_by_gidnumber($gidnumber)
	{
		if ($this->_lc->search("ou=UnixGroups,ou=Groups,dc=coinnetwork,dc=dl", "gidNumber=".$gidnumber, $this->_ent))
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Give the GID corresponding to a GID Number
	*
	* @access	public
	* @param	integer		$gidnumber	GID Number of the User
	* @return	string
	*/
	function get_gid_from_gidnumber($gidnumber)
	{
		if ($this->_lc->search("ou=UnixGroups,ou=Groups,dc=coinnetwork,dc=dl", "gidNumber=".$gidnumber, $this->_ent, array('cn')))
		{
			$this->_entry_to_attrs(0);
			return $this->get_attr_value('cn');
		}

		return false;
	}

	/**
	* Give the GID Number corresponding to a GID
	*
	* @access	public
	* @param	string		$gid		GID Number of the User
	* @return	integer
	*/
	function get_gidnumber_from_gid($gid)
	{
		if ($this->_lc->search("ou=UnixGroups,ou=Groups,dc=coinnetwork,dc=dl", "cn=".$gid, $this->_ent, array('gidnumber')))
		{
			$this->_entry_to_attrs(0);
			return $this->get_attr_value('gidnumber');
		}

		return false;
	}

	/**
	* Get a Group list
	*
	* Always fetched attributs : objectclass, cn
	*
	* @access	public
	* @param	array		$attr		Complementary attributs to fetch
	* @param	string		$crit		Alternative criterias
	* @return	interger
	*/
	function get_list($attr = array(), $crit = "")
	{
		if (!is_array($attr))
			$attr = array();
		$attr = array_merge(array('objectclass', 'cn', 'gidnumber'), $attr);
		if (!$crit)
			$crit = "cn=*";

		$ret = $this->_lc->search("ou=UnixGroups,ou=Groups,dc=coinnetwork,dc=dl", $crit, $this->_ent, $attr, false);

		if ($ret)
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Give the first unused GID Number
	*
	* @access	public
	* @return	integer
	*/
	function get_first_free_gidnumber()
	{
		$ret = $this->_lc->search("ou=UnixGroups,ou=Groups,dc=coinnetwork,dc=dl", "cn=*", $this->_ent, array('gidnumber'), false);

		if ($ret)
		{
			$z = array();
			for($i=0; $i<$this->_ent->get_count(); $i++)
			{
				$this->_entry_to_attrs($i);
				$z[$i] = $this->get_attr_value('gidnumber');
			}
			for ($i=5000; $i<=65535; $i++)
			{
				if (!in_array($i, $z))
					return $i;
			}
		}

		return false;
	}

	/**
	* Do what need to be done before applying changes (OVERLOAD)
	*
	* @access	private
	*/
	function _pre_commit_changes()
	{
		if ($this->has_attr_changed('userpassword'))
		{
			if (substr($this->get_attr_value('userpassword'), 0, 7)!="{crypt}")
			{
				$this->_set_attr_value('userpassword', _unix_crypt($this->get_attr_value('userpassword')));
			}
		}
	}

}

?>
