<?php
/*

Copyright (c)2002-2003 DuckCorp(tm) and RtpNet(tm)



This file is part of LMS.

LMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

LMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LMS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/**
* User Type : unknown
*
* @const LMS_UTYPE_UNKNOWN
*/
define('LMS_UTYPE_UNKNOWN', 0);
/**
* User Type : admin
*
* @const LMS_UTYPE_ADMIN
*/
define('LMS_UTYPE_ADMIN', 1);
/**
* User Type : common
*
* @const LMS_UTYPE_COMMON
*/
define('LMS_UTYPE_COMMON', 2);
/**
* User Type : service
*
* @const LMS_UTYPE_SERVICE
*/
define('LMS_UTYPE_SERVICE', 3);

/**
* LMS User Manager CLass
*
* @access	public
*/
class LmsUserManager extends LmsObjectManager
{

	/**
	* Constructor
	*
	* @access	public
	* @param	object LmsLdapConnection	&$ldap_connection
	*/
	function LmsUserManager(&$ldap_connection)
	{
		LmsObjectManager::LmsObjectManager(&$ldap_connection);

		$this->_dn_attr = "uid";
		$this->_root_dn = "ou=Users,dc=coinnetwork,dc=dl";
		$this->_propagation_key_list = array('memberUid', 'ownerUID', 'adminUID');
		$this->_check_key_list = array('objectclass', 'homephone', 'mobile', 'mail');


		$this->_register_attr("UID", "uid", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("Common Name", "cn", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("UID Number", "uidnumber", LMS_ATYPE_INT, null, 1, true);
		$this->_register_attr("GID Number", "gidnumber", LMS_ATYPE_INT, null, 1, true);
		$this->_register_attr("Surname", "sn", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Given Name", "givenname", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Nickname", "nn", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Home Directory", "homedirectory", LMS_ATYPE_STR, 60, 1, false);
		$this->_register_attr("Password", "userpassword", LMS_ATYPE_STR, 100, 1, false);
		$this->_register_attr("Shell", "loginshell", LMS_ATYPE_STR, 60, 1, false);
		$this->_register_attr("Gecos", "gecos", LMS_ATYPE_STR, 100, 1, false);
		$this->_register_attr("Description", "description", LMS_ATYPE_STR, 300, 1, false);
		$this->_register_attr("Days since Jan 1, 1970 that password was last changed", "shadowlastchange", LMS_ATYPE_INT, null, 1, false, false);
		$this->_register_attr("Days before Password may be changed", "shadowmin", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Days before Password must be changed", "shadowmax", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Days before password is to expire that user is warned", "shadowwarning", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Days after password expires that account is disabled", "shadowinactive", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Days since Jan 1, 1970<BR>that account is disabled", "shadowexpire", LMS_ATYPE_INT, null, 1, false, false);
		$this->_register_attr("Account is disactivated", "isdisactivated", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Contribution is paid", "contributionispaid", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Forbid profile modification", "profilemodifforbiden", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Birthday", "birthday", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Occupation", "occupation", LMS_ATYPE_STR, 60, 1, false);
		$this->_register_attr("Room Number", "roomnumber", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Home Address", "homepostaladdress", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Home Telephone", "homephone", LMS_ATYPE_STR, 15, 1, false);
		$this->_register_attr("Mobile Telephone", "mobile", LMS_ATYPE_STR, 15, 1, false);
		$this->_register_attr("Mail Alias", "mailalias", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Mails", "mail", LMS_ATYPE_STR, 30, 10, false);
		$this->_register_attr("icq uin", "icq", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Yahoo Messenger nick", "ym", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("AOL Instant Messenger nick", "aim", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Favourite Drink", "drink", LMS_ATYPE_STR, 30, 1, false);
		
		register_shutdown_function(array(&$this, "_LmsUserManager"));
	}

	/**
	* Destructor
	*
	* @access	public
	*/
	function _LmsUserManager()
	{
	}

	/**
	* Give User Type
	*
	* @access	public
	* @return	integer
	*/
	function get_user_type()
	{
		if (!is_array($this->_attrs['objectclass']['value']))
			return LMS_UTYPE_UNKNOWN;

		if (in_array('coinnetworkAdminUser', $this->_attrs['objectclass']['value']))
			return LMS_UTYPE_ADMIN;
		if (in_array('coinnetworkCommonUser', $this->_attrs['objectclass']['value']))
			return LMS_UTYPE_COMMON;
		if (in_array('coinnetworkServiceUser', $this->_attrs['objectclass']['value']))
			return LMS_UTYPE_SERVICE;

		return LMS_UTYPE_UNKNOWN;
	}

	/**
	* Find a User with his UID
	*
	* @access	public
	* @param	string		$uid		UID of the User
	* @return	boolean		$ok		True if successful
	*/
	function find_by_uid($uid)
	{
		if ($this->_lc->search($this->_root_dn, "uid=".$uid, $this->_ent))
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Find a User with his UID Number
	*
	* @access	public
	* @param	integer		$uidnumber	UID Number of the User
	* @return	boolean		$ok		True if successful
	*/
	function find_by_uidnumber($uidnumber)
	{
		if ($this->_lc->search($this->_root_dn, "uidNumber=".$uid, $this->_ent))
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Give the UID corresponding to an UID Number
	*
	* @access	public
	* @param	integer		$uidnumber	UID Number of the User
	* @return	string
	*/
	function get_uid_from_uidnumber($uidnumber)
	{
		if ($this->_lc->search($this->_root_dn, "uidNumber=".$uid, $this->_ent, array('uid')))
		{
			$this->_entry_to_attrs(0);
			return $this->get_attr_value('uid');
		}

		return false;
	}

	/**
	* Give the UID Number corresponding to an UID
	*
	* @access	public
	* @param	string		$uid		UID Number of the User
	* @return	interger
	*/
	function get_uidnumber_from_uid($uid)
	{
		if ($this->_lc->search($this->_root_dn, "uid=".$uid, $this->_ent, array('uidnumber')))
		{
			$this->_entry_to_attrs(0);
			return $this->get_attr_value('uidnumber');
		}

		return false;
	}

	/**
	* Get a User list
	*
	* Always fetched attributs : objectclass, uid, cn, sn, givenname, nn
	*
	* @access	public
	* @param	integer		$c		User Type to list (0 => every users)
	* @param	array		$attr		Complementary attributs to fetch
	* @param	string		$crit		Alternative criterias
	* @return	interger
	*/
	function get_list($c, $attr = array(), $crit = "")
	{
		if (!is_array($attr))
			$attr = array();
		$attr = array_merge($this->_check_key_list, $attr);
		$attr = array_merge(array('uid', 'uidnumber', 'cn', 'sn', 'givenname', 'nn'), $attr);
		if (!$crit)
			$crit = "uid=*";
		switch ($c)
		{
		case LMS_UTYPE_UNKNOWN:
			$ret = $this->_lc->search($this->_root_dn, $crit, $this->_ent, $attr);
			break;
		case LMS_UTYPE_ADMIN:
			$ret = $this->_lc->search("ou=AdminUsers,".$this->_root_dn, $crit, $this->_ent, $attr, false);
			break;
		case LMS_UTYPE_COMMON:
			$ret = $this->_lc->search("ou=CommonUsers,".$this->_root_dn, $crit, $this->_ent, $attr, false);
			break;
		case LMS_UTYPE_SERVICE:
			$ret = $this->_lc->search("ou=ServiceUsers,".$this->_root_dn, $crit, $this->_ent, $attr, false);
			break;
		default:
			return false;
		}

		if ($ret)
		{
			$this->_entry_to_attrs(0);
			return $this->_ent->get_count();
		}

		return false;
	}

	/**
	* Give the first unused UID Number
	*
	* @access	public
	* @param	integer		$c		User Type to list (0 => every users)
	* @return	integer
	*/
	function get_first_free_uidnumber($c)
	{
		switch ($c)
		{
		case LMS_UTYPE_UNKNOWN:
		case LMS_UTYPE_ADMIN:
		case LMS_UTYPE_COMMON:
			$min = 10000;
			$max = 65535;
			break;
			break;
		case LMS_UTYPE_SERVICE:
			$min = 5000;
			$max = 10000;
			break;
		default:
			return false;
		}

		$ret = $this->_lc->search($this->_root_dn, "uid=*", $this->_ent, array('uidnumber'), false);

		if ($ret)
		{
			$z = array();
			for($i=0; $i<$this->_ent->get_count(); $i++)
			{
				$this->_entry_to_attrs($i);
				$z[$i] = $this->get_attr_value('uidnumber');
			}
			for ($i=$min; $i<=$max; $i++)
			{
				if (!in_array($i, $z))
					return $i;
			}
		}

		return false;
	}

	/**
	* Do what need to be done before applying changes (OVERLOAD)
	*
	* @access	private
	*/
	function _pre_commit_changes()
	{
		$user_type = $this->get_user_type();
		if ($user_type==LMS_UTYPE_SERVICE)
		{
			if ($this->has_attr_changed('uid'))
				$this->_set_attr_value('cn', $this->get_attr_value('uid'));
			$this->_set_attr_value('initials', null);
		}
		else
		{
			if ($this->has_attr_changed('sn') || $this->has_attr_changed('givenname'))
			{
				$this->_set_attr_value('cn', $this->get_attr_value('sn')." ".$this->get_attr_value('givenname'));
				$this->_set_attr_value('initials', strtoupper(substr($this->get_attr_value('sn'), 0, 1)." ".substr($this->get_attr_value('givenname'), 0, 1)));
			}
		}

		if ($this->has_attr_changed('userpassword'))
		{
			if (substr($this->get_attr_value('userpassword'), 0, 7)!="{crypt}")
			{
				$this->_set_attr_value('userpassword', _unix_crypt($this->get_attr_value('userpassword')));
			}
		}

		if (!$this->get_attr_value('homedirectory'))
			$this->_set_attr_value('homedirectory', "/home/".$this->get_attr_value('uid'));

		if (!$this->get_attr_value('uidnumber'))
			$this->_set_attr_value('uidnumber', $this->get_first_free_uidnumber($user_type));
	}

	/**
	* Additionnal Check (OVERLOAD)
	*
	* @access	private
	* @return	boolean		$ok		True if successful
	*/
	function _check_changes($mod = array())
	{
		$user_type = $this->get_user_type();
		if ($user_type==LMS_UTYPE_ADMIN)
		{
			if (!$this->get_attr_value('homephone') && !$this->get_attr_value('mobile'))
				return false;
			if (count($this->get_attr_value('mail'))==0)
				return false;
		}

		return LmsObjectManager::_check_changes($mod);
	}

	/**
	* Return a sub DN if needed (OVERLOAD)
	*
	* @access	public
	* @return	string
	*/
	function _get_sub_dn()
	{
		$user_type = $this->get_user_type();

		switch($user_type)
		{
		case LMS_UTYPE_SERVICE:
			return "ou=ServiceUsers";
		case LMS_UTYPE_ADMIN:
			return "ou=AdminUsers";
		default:
			return "ou=CommonUsers";
		}
	}

	/**
	* Change User Type
	*
	* @access	public
	* @param	integer		$new_user_type		Type of User
	* @return	boolean		$ok		True if successful
	*/
	function set_user_type($new_user_type)
	{
		if ($this->_ent->get_count() != 0)
		{
			$user_type = $this->get_user_type();

			if ($new_user_type==$user_type)
				return false;
			if (($user_type==LMS_UTYPE_SERVICE) || ($new_user_type==LMS_UTYPE_SERVICE))
				return false;
		}

		return $this->_set_user_type($new_user_type);
	}

	/**
	* Change User Type (internal use)
	*
	* @access	private
	* @param	integer		$new_user_type		Type of User
	* @return	boolean		$ok		True if successful
	*/
	function _set_user_type($new_user_type = null, $set_changed = true)
	{
		if (!$new_user_type)
			$new_user_type = $this->get_user_type();

		switch($new_user_type)
		{
		case LMS_UTYPE_ADMIN:
			$this->_attrs['objectclass']['value'] = array('top', 'posixAccount', 'shadowAccount', 'coinnetworkAdminUser');
			$this->_activate_attrs(array('nn', 'initials', 'roomNumber', 'homePostalAddress', 'homeTelephoneNumber', 'mobileTelephoneNumber', 'mail', 'favouriteDrink', 'suscribedService', 'icq', 'ym', 'aim', 'occupation', 'mailAlias', 'birthday'));
			$this->_desactivate_attrs(array('contributionispaid', 'profilemodifforbiden'));
			break;
		case LMS_UTYPE_COMMON:
			$this->_attrs['objectclass']['value'] = array('top', 'posixAccount', 'shadowAccount', 'coinnetworkCommonUser');
			$this->_activate_attrs(array('nn', 'initials', 'roomNumber', 'homePostalAddress', 'homeTelephoneNumber', 'mobileTelephoneNumber', 'mail', 'favouriteDrink', 'suscribedService', 'contributionIsPaid', 'icq', 'ym', 'aim', 'occupation', 'mailAlias', 'profileModifForbiden', 'birthday'));
			break;
		case LMS_UTYPE_SERVICE:
			$this->_attrs['objectclass']['value'] = array('top', 'posixAccount', 'shadowAccount', 'coinnetworkServiceUser');
			$this->_desactivate_attrs(array('nn', 'initials', 'roomNumber', 'homePostalAddress', 'homeTelephoneNumber', 'mobileTelephoneNumber', 'mail', 'favouriteDrink', 'suscribedService', 'contributionIsPaid', 'icq', 'ym', 'aim', 'occupation', 'mailAlias', 'profileModifForbiden', 'birthday'));
			break;
		default:
			return false;
		}

		if ($set_changed)
			$this->_attrs['objectclass']['changed'] = true;

		return true;
	}

	/**
	* Do Object need a renaming before saving changes ? (OVERLOAD)
	*
	* @access	private
	* @return	boolean
	*/
	function _need_renaming()
	{
		return $this->_attrs['objectclass']['changed'];
	}

	/**
	* Convert an LDAP Entry to Attributs (OVERLOAD)
	*
	* @access	private
	* @param	integer		$i		Number of the Entry to convert
	*/
	function _entry_to_attrs($i)
	{
		LmsObjectManager::_entry_to_attrs($i);

		$this->_set_user_type(null, false);
	}

}

?>
